<?php

require_once dirname(__FILE__).'/../lib/articleGeneratorConfiguration.class.php';
require_once dirname(__FILE__).'/../lib/articleGeneratorHelper.class.php';

/**
 * article actions.
 *
 * @package    sf_sandbox
 * @subpackage article
 * @author     Your name here
 * @version    SVN: $Id: actions.class.php 23810 2009-11-12 11:07:44Z Kris.Wallsmith $
 */
class articleActions extends autoArticleActions
{

  public function executeDelete(sfWebRequest $request)
  {
    $request->checkCSRFProtection();

    $this->dispatcher->notify(new sfEvent($this, 'admin.delete_object', array('object' => $this->getRoute()->getObject())));

    try {
    if ($this->getRoute()->getObject()->delete())
    {
      $this->getUser()->setFlash('notice', 'The item was deleted successfully.');
    }
    }catch (Doctrine_Connection_Mysql_Exception $e) {
      $this->getUser()->setFlash('error', 'Vous ne pouvez pas supprimer cet article.');
    }

    $this->redirect('@article');
  }
	public function executeEdit(sfWebRequest $request)
	{
		$this->article = $this->getRoute()->getObject();

		$this->article->setPrixVenteHT(Utils::doubleEnToFr($this->article->getPrixVenteHT()));
		$this->article->setTauxRemiseFournisseur(Utils::doubleEnToFr($this->article->getTauxRemiseFournisseur()));
		$this->article->setPrixVenteTHS(Utils::doubleEnToFr($this->article->getPrixVenteTHS()));
		$this->article->setMargePourcentage(Utils::doubleEnToFr($this->article->getMargePourcentage()));
		$this->article->setMargeEuro(Utils::doubleEnToFr($this->article->getMargeEuro()));
		$this->article->setPrixTTC(Utils::doubleEnToFr($this->article->getPrixTTC()));

		$this->form = $this->configuration->getForm($this->article);
	}

	protected function processForm(sfWebRequest $request, sfForm $form)
	{
		$values = $request->getParameter($form->getName());
		$values["prix_vente_ht"] = Utils::doubleFrToEn($values["prix_vente_ht"]);
		$values["taux_remise_fournisseur"] = Utils::doubleFrToEn($values["taux_remise_fournisseur"]);
		$values["prix_vente_ths"] = Utils::doubleFrToEn($values["prix_vente_ths"]);
		$values["marge_pourcentage"] = Utils::doubleFrToEn($values["marge_pourcentage"]);
		$values["marge_euro"] = Utils::doubleFrToEn($values["marge_euro"]);
		$values["prix_ttc"] = Utils::doubleFrToEn($values["prix_ttc"]);
		
		$form->bind($values, $request->getFiles($form->getName()));
		if ($form->isValid())
		{
			// Balise meta
			$famille = Doctrine::getTable("Famille")->find($values["id_famille"]);
			$values["balise_meta"] = $famille->getLibelle().' - '.$values["designation_detaillee"];
		
			$form->bind($values, $request->getFiles($form->getName()));
			
			$notice = $form->getObject()->isNew() ? 'The item was created successfully.' : 'The item was updated successfully.';

			try
			{
				$article = $form->save();
			} 
			catch (Doctrine_Validator_Exception $e) 
			{

				$errorStack = $form->getObject()->getErrorStack();

				$message = get_class($form->getObject()) . ' has ' . count($errorStack) . " field" . (count($errorStack) > 1 ?  's' : null) . " with validation errors: ";
				foreach ($errorStack as $field => $errors)
				{
					$message .= "$field (" . implode(", ", $errors) . "), ";
				}
				$message = trim($message, ', ');

				$this->getUser()->setFlash('error', $message);
				return sfView::SUCCESS;
			}

			$this->dispatcher->notify(new sfEvent($this, 'admin.save_object', array('object' => $article)));

			if ($request->hasParameter('_save_and_add'))
			{
				$this->getUser()->setFlash('notice', $notice.' You can add another one below.');

				$this->redirect('@article_new');
			}
			else
			{
				$this->getUser()->setFlash('notice', $notice);

				$this->redirect(array('sf_route' => 'article_edit', 'sf_subject' => $article));
			}
		}
		else
		{
			$this->getUser()->setFlash('error', 'The item has not been saved due to some errors.', false);
		}
	}

	public function executeIndex(sfWebRequest $request)
	{
		$this->max_per_page = 1000;
		if($this->getUser()->hasAttribute('max_per_page_article'))
		{
			if($this->configuration->getPagerMaxPerPage() != $this->getUser()->getAttribute('max_per_page_article'))
			{
				$this->setPage(1);
			}
			
			$this->max_per_page = $this->getUser()->getAttribute('max_per_page_article');
		}
		
		$this->configuration->setPagerMaxPerPage($this->max_per_page);

		// sorting
		if ($request->getParameter('sort') && $this->isValidSortColumn($request->getParameter('sort')))
		{
			$this->setSort(array($request->getParameter('sort'), $request->getParameter('sort_type')));
		}
		
		// pager
		if ($request->getParameter('page'))
		{
			$this->setPage($request->getParameter('page'));
		}

		$this->pager = $this->getPager();
		$this->sort = $this->getSort();
			
		// Formulaire article import
		$this->form = new ArticleImportForm();
		
		// Permet de récupérer la valeur de reference_ths_sage pour le filtre
		$filters = $this->getFilters();
		if(isset($filters['reference_ths_sage']))
		{
			$this->filters->setDefault('reference_ths_sage', $filters['reference_ths_sage']);
		}
	}
	
	public function executeUpdateReferenceThs(sfWebRequest $request)
	{
		if ($request->isMethod("POST")) {
			$form = new ArticleImportForm();
			$form->bind($request->getParameter('article_import'), $request->getFiles('article_import'));
			
			if ($form->isValid()) {
				$file = $form->getValue('file');

				$filename = 'uploaded_'.sha1($file->getOriginalName());
				$extension = $file->getExtension($file->getOriginalExtension());
				$file->save(sfConfig::get('sf_upload_dir').'/'.$filename.$extension);
				
				// Récupère les données
				$fichier = sfConfig::get('sf_upload_dir').'/'.$filename.$extension;
				$connection = Doctrine_Manager::connection();

				try {
				    $connection->beginTransaction();
				    
				    if (($handle = fopen($fichier, "r")) !== FALSE) {
					while (($data = fgetcsv($handle, 0, ";")) !== FALSE) {
					    $objet = Doctrine_Core::getTable("Article")->find($data[0]);
					    
					    if ($objet) {
						$objet->reference_ths = $data[1];
						$objet->save();
					    }

					}
				    }
							
				    fclose($handle);
				    $connection->commit();
				    $this->getUser()->setFlash('notice', 'La modification des références s\'est déroulé avec succès');
				} catch (Exception $e) {
					$connection->rollback();
					$this->getUser()->setFlash('error', $e->getMessage().$e->getTraceAsString());
				}
				
				unlink(sfConfig::get('sf_upload_dir').'/'.$filename.$extension);
			} else {
				$this->getUser()->setFlash('error', 'Erreur lors de l\'importation');
			}
		}
		
		$this->redirect("@article");
	}
	
	public function executeImportReporting(sfWebRequest $request)
	{
		if ($request->isMethod("POST")) {
			$form = new ArticleImportForm();
			$form->bind($request->getParameter('article_import'), $request->getFiles('article_import'));
			
			if ($form->isValid()) {
				$file = $form->getValue('file');

				$filename = 'uploaded_'.sha1($file->getOriginalName());
				$extension = $file->getExtension($file->getOriginalExtension());
				$file->save(sfConfig::get('sf_upload_dir').'/'.$filename.$extension);

				// Récupère la liste des familles d'articles
				$listeFamilleArticle = FamilleTable::recupererListeArticleFamilleEnTableau();

				//Récupère les données
				$fichier = sfConfig::get('sf_upload_dir').'/'.$filename.$extension;

				try {
					// Vérification que les images existent bien
					$repertoireImage = sfConfig::get('import_dir').DIRECTORY_SEPARATOR;
					$repertoireArticle = sfConfig::get('article_image_dir').DIRECTORY_SEPARATOR;
					$repertoireArticleMiniature = sfConfig::get('article_miniature_dir').DIRECTORY_SEPARATOR;

					if (($handle = fopen($fichier, "r")) !== FALSE) {
						while (($data = fgetcsv($handle, 0, ";")) !== FALSE) {
							if (!file_exists($repertoireImage.$data[26].".jpg")) {
								//throw new Exception("Aucune image n'est associé à la référence THS '$data[26]' ".$repertoireImage.$data[26].".jpg");
							}
						}
						
						fclose($handle);
					}

					// Vérification que tous les articles ont une famille
					if (($handle = fopen($fichier, "r")) !== FALSE) {
						while (($data = fgetcsv($handle, 0, ";")) !== FALSE) {
							if ($data[27] == "") {
								throw new Exception("La référence THS '$data[26]' ne contient pas de famille");
							}

							if (!isset($listeFamilleArticle[strtolower($data[27])])) {
								throw new Exception("La référence THS '$data[26]' contient une famille qui n'existe pas");
							}
						}
						
						fclose($handle);
					}

					// Vérification des références
					$listeReference = array();
					$listeReferenceTHS = array();
					
					if (($handle = fopen($fichier, "r")) !== FALSE) {
						while (($data = fgetcsv($handle, 0, ";")) !== FALSE) {
							$listeReferenceTHS[] = $data[26];
							$listeReference[$data[26]] = $data[3];
						}
						
						fclose($handle);
					}

					try {

						if (sizeof($listeReferenceTHS) > 0) {
							// On récupère toutes les références existantes et on compare le couple reference_ths / reference_ths_sage
							// Si un couple ne correspond, on termine l'import
							$listeArticle = ArticleTable::recupererListeArticlePourImport($listeReferenceTHS);
							
							foreach($listeArticle as $article) {
								if ($listeReference[$article->reference_ths] != $article->reference_ths_sage) {
									throw new Exception("La référence THS '$article->reference_ths' ne contient pas la même référence sage");
								}
							}
						}

						if (($handle = fopen($fichier, "r")) !== FALSE) {
							$nouvellesReferences = array();
							$modifies = array();
							$nonModifies = array();
							
							$listeReferenceFournisseur = array();
							$listeReferenceTHSSage = array();
							$listeReferenceTHSSite = array();
							
							$referenceFournisseurDoublon = array();
							$referenceTHSSageDoublon = array();
							$referenceTHSSiteDoublon = array();
							
							$champsObligatoire = array();
							
							$line = 1;
								
							while (($data = fgetcsv($handle, 0, ";")) !== FALSE) {
								// On vérifie si le fournisseur existe
								$fournisseur = Doctrine_Core::getTable("Fournisseur")->findOneByCode($data[1]);
								
								if (!($fournisseur instanceof Fournisseur)) {
								    throw new Exception("Le code fournisseur '$data[1]' n'existent pas en base de données");
								}
								
								// Vérification champs obligatoire
								// A, B, C, D, E, F, G, K
								
								if ($data[0] == "" 
								    || $data[1] == ""
								    || $data[2] == ""
								    || $data[3] == ""
								    || $data[4] == ""
								    || $data[5] == ""
								    || $data[6] == ""
								    || $data[10] == ""
								) {
								    $champsObligatoire[$line]['data'] = $data;
								    $champsObligatoire[$line]['fields'] = array();
								    
								    if ($data[0] == "") {
									$champsObligatoire[$line]['fields'][] = "A";
								    }
								    if ($data[1] == "") {
									$champsObligatoire[$line]['fields'][] = "B";
								    }
								    if ($data[2] == "") {
									$champsObligatoire[$line]['fields'][] = "C";
								    }
								    if ($data[3] == "") {
									$champsObligatoire[$line]['fields'][] = "D";
								    }
								    if ($data[4] == "") {
									$champsObligatoire[$line]['fields'][] = "E";
								    }
								    if ($data[5] == "") {
									$champsObligatoire[$line]['fields'][] = "F";
								    }
								    if ($data[6] == "") {
									$champsObligatoire[$line]['fields'][] = "G";
								    }
								    if ($data[10] == "") {
									$champsObligatoire[$line]['fields'][] = "K";
								    }
								    
								    $line++;
								    continue;
								}
								
								// Calcul du prix
								$prix = Utils::doubleFrToEn($data[11]);
								$coefficient = FournisseurCoefficientTable::findCoefficientByCodeFournisseurAndPrice($data[1], $prix);

								if (count($coefficient) > 1) {
								    die("Il y a plusieurs coefficient pour le fournisseur ".$data[1]." et le montant ".$data[11]);
								} elseif (count($coefficient) == 1) {
								    $coeff = $coefficient[0]["coefficient"];
								} else {
								    $coeff = 1;
								}
								
								// Calcul du coeff de vente ths, marge ths en % et €
								/**
								 * H = F - ( F * G / 100 )
								 * 7 = 5 - ( 5 * 6 / 100 )
								 */
								$f = Utils::doubleFrToEn($data[5]);
								$g = Utils::doubleFrToEn($data[6]);
								$data[7] = Utils::doubleEnToFr(round($f-($f*$g/100), 2));
								$h = Utils::doubleFrToEn($data[7]);
								
								$data[8] = Utils::doubleEnToFr(round($prix/$h, 2));
								$data[12] = Utils::doubleEnToFr(round($prix-$h, 2));
								$data[9] = Utils::doubleEnToFr(round(Utils::doubleFrToEn($data[12])*100/$prix, 2));
								
								// Calcul des données avec le coefficient de vente THS
								$prix_web = $prix * $coeff;
								$data[14] = Utils::doubleEnToFr(round($prix_web, 2));
								$o = round($prix_web, 2);
								
								/**
								 * P = O / H
								 * 15 = 14 / 7;
								 */
								$data[15] = Utils::doubleEnToFr(round($o/$h, 2));
								
								/**
								 * R = O - H
								 * 17 = 14 - 7
								 */
								$data[17] = Utils::doubleEnToFr(round($o-$h, 2));
								$r = round($o-$h, 2);
								
								/**
								 * Q = R x 100 / O 
								 * 16 = 17 * 100 / 14
								 */
								$data[16] = Utils::doubleEnToFr(round($r*100/$o, 2));
								
							    
								$objet = Doctrine_Core::getTable("Article")->find($data[26]);
								
								// On vérifie si dans le fichier il n'y a pas de doublon sur la référence fournisseur
								$referenceFournisseurDoublon[$data[0]]['reference'] = $data[0];
								$referenceFournisseurDoublon[$data[0]]['data'][$line] = $data;
								
								// On vérifie si dans le fichier il n'y a pas de doublon sur la référence THS Sage
								$referenceTHSSageDoublon[$data[3]]['reference'] = $data[3];
								$referenceTHSSageDoublon[$data[3]]['data'][$line] = $data;
								
								// On vérifie si dans le fichier il n'y a pas de doublon sur la référence THS Site
								$referenceTHSSiteDoublon[$data[26]]['reference'] = $data[26];
								$referenceTHSSiteDoublon[$data[26]]['data'][$line] = $data;
								
								if (!($objet instanceof Article)) {
									$objet = new Article();
									$nouvellesReferences[] = $data;
								} else {
									$modif = array(
									    'designation' => false,
									    'prix' => false
									);
									
									if ($objet->designation_courte != $data[29] || $objet->designation_detaillee != $data[28]) {
										$modif['designation'] = true;
										$modif['designation_courte_old'] = $objet->designation_courte;
										$modif['designation_detaillee_old'] = $objet->designation_detaillee;
										$modif['data'] = $data;
										$modifies[] = $modif;
									} else {
										$nonModifies[] = $data;
									}
								}
								
								$line++;
							}
							
							/**
							 * Suppression des références qui ne sont pas en doublon
							 */
							foreach ($referenceFournisseurDoublon as $key => $reference) {
							    if (count($reference['data']) <= 1) {
								unset($referenceFournisseurDoublon[$key]);
							    }
							}
							
							foreach ($referenceTHSSageDoublon as $key => $reference) {
							    if (count($reference['data']) <= 1) {
								unset($referenceTHSSageDoublon[$key]);
							    }
							}
							
							foreach ($referenceTHSSiteDoublon as $key => $reference) {
							    if (count($reference['data']) <= 1) {
								unset($referenceTHSSiteDoublon[$key]);
							    }
							}
							
							fclose($handle);
						}
					} catch(Exception $e) {
						throw new Exception($e->getMessage());
					}
				} catch(Exception $e) {
					unlink(sfConfig::get('sf_upload_dir').'/'.$filename.$extension);
					$this->getUser()->setFlash('error', $e->getMessage());
					$this->redirect("@article");
				}
			} else {
				$this->getUser()->setFlash('error', 'Erreur lors de l\'importation');
			}
		}
		
		$this->champsObligatoire = $champsObligatoire;
		$this->referenceFournisseurDoublon = $referenceFournisseurDoublon;
		$this->referenceTHSSageDoublon = $referenceTHSSageDoublon;
		$this->referenceTHSSiteDoublon = $referenceTHSSiteDoublon;
		$this->nouvellesReferences = $nouvellesReferences;
		$this->modifies = $modifies;
		$this->nonModifies = $nonModifies;
		$this->fichier = sfConfig::get('sf_upload_dir').'/'.$filename.$extension;
	}
	
	public function executeExport(sfWebRequest $request)
	{
		if ($request->isMethod("POST")) {
			$fichier = $request->getParameter('fichier');
			
			
			$new_file = "export.csv";
			$fp = fopen(sfConfig::get('sf_upload_dir').'/'.$new_file, "w+");
			
			if (($handle = fopen($fichier, "r")) !== FALSE) {
				while (($data = fgetcsv($handle, 0, ";")) !== FALSE) {
				    // Vérification champs obligatoire
				    // A, B, C, D, E, F, G, K
				    if ($data[0] == "" 
					|| $data[1] == ""
					|| $data[2] == ""
					|| $data[3] == ""
					|| $data[4] == ""
					|| $data[5] == ""
					|| $data[6] == ""
					|| $data[10] == ""
				    ) {
					continue;
				    }
				    
				    // On récupère le coefficient
				    $prix = Utils::doubleFrToEn($data[11]);
				    $coefficient = FournisseurCoefficientTable::findCoefficientByCodeFournisseurAndPrice($data[1], $prix);

				    if (count($coefficient) > 1) {
					die("Il y a plusieurs coefficient pour le fournisseur ".$data[1]." et le montant ".$data[11]);
				    } elseif (count($coefficient) == 1) {
					$coeff = $coefficient[0]["coefficient"];
				    } else {
					$coeff = 1;
				    }
								
				    // Calcul du coeff de vente ths, marge ths en % et €
				    /**
				     * H = F - ( F * G / 100 )
				     * 7 = 5 - ( 5 * 6 / 100 )
				     */
				    $f = Utils::doubleFrToEn($data[5]);
				    $g = Utils::doubleFrToEn($data[6]);
				    $data[7] = Utils::doubleEnToFr(round($f-($f*$g/100), 2));
				    $h = Utils::doubleFrToEn($data[7]);

				    $data[8] = Utils::doubleEnToFr(round($prix/$h, 2));
				    $data[12] = Utils::doubleEnToFr(round($prix-$h, 2));
				    $data[9] = Utils::doubleEnToFr(round(Utils::doubleFrToEn($data[12])*100/$prix, 2));

				    // Calcul des données avec le coefficient de vente THS
				    $prix_web = $prix * $coeff;
				    $data[14] = Utils::doubleEnToFr(round($prix_web, 2));
				    $o = round($prix_web, 2);

				    /**
				     * P = O / H
				     * 15 = 14 / 7;
				     */
				    $data[15] = Utils::doubleEnToFr(round($o/$h, 2));

				    /**
				     * R = O - H
				     * 17 = 14 - 7
				     */
				    $data[17] = Utils::doubleEnToFr(round($o-$h, 2));
				    $r = round($o-$h, 2);

				    /**
				     * Q = R x 100 / O 
				     * 16 = 17 * 100 / 14
				     */
				    $data[16] = Utils::doubleEnToFr(round($r*100/$o, 2));
				    
				    fputcsv($fp, $data, ";");
				}

			}
			
		    fclose($fp);
			$new_fichier = sfConfig::get('sf_upload_dir').'/'.$new_file;
			readfile($new_fichier);
			$type = mime_content_type($new_fichier);
			header('Content-Type: application/force-download');
			header('Content-Transfer-Encoding: '.$type.'\n');
			header('Content-Length: '. filesize($new_fichier));
			header('Content-disposition: attachment; filename=export.csv');
			header('Pragma: no-cache');
			header('Cache-Control: no-store, no-cache, must-revalidate, post-check=0, pre-check=0');
			header('Expires: 0');
			die;
		}
	}
	
	public function executeImportV2(sfWebRequest $request)
	{
		if ($request->isMethod("POST")) {
			$form = new ArticleImportForm();
			$form->bind($request->getParameter('article_import'), $request->getFiles('article_import'));
			
			if ($form->isValid()) {
				$file = $form->getValue('file');

				$filename = 'uploaded_'.sha1($file->getOriginalName());
				$extension = $file->getExtension($file->getOriginalExtension());
				$file->save(sfConfig::get('sf_upload_dir').'/'.$filename.$extension);

				// Récupère la liste des familles d'articles
				$listeFamilleArticle = FamilleTable::recupererListeArticleFamilleEnTableau();

				//Récupère les données
				$fichier = sfConfig::get('sf_upload_dir').'/'.$filename.$extension;

				try {
					// Vérification que les images existent bien
					$repertoireImage = sfConfig::get('import_dir').DIRECTORY_SEPARATOR;
					$repertoireArticle = sfConfig::get('article_image_dir').DIRECTORY_SEPARATOR;
					$repertoireArticleMiniature = sfConfig::get('article_miniature_dir').DIRECTORY_SEPARATOR;

					if (($handle = fopen($fichier, "r")) !== FALSE) {
						while (($data = fgetcsv($handle, 0, ";")) !== FALSE) {
							if (!file_exists($repertoireImage.$data[23].".jpg") && !file_exists($repertoireImage.$data[26].".jpg") && !file_exists($repertoireImage.$data[0].".jpg")) {
								$objet = Doctrine_Core::getTable("Article")->findOneByReferenceThsSage($data[3]);

								if (($objet instanceof Article) && file_exists($repertoireImage.$objet->getReferenceThs().".jpg")) {
									rename($repertoireImage.$objet->getReferenceThs().".jpg", $repertoireImage.$data[26].".jpg");
									rename($repertoireArticle.$objet->getReferenceThs()."_100.jpg", $repertoireArticle.$data[26]."_100.jpg");
									rename($repertoireArticleMiniature.$objet->getReferenceThs()."_100.jpg", $repertoireArticleMiniature.$data[26]."_100.jpg");
								} else {
									throw new Exception("Aucune image n'est associé à la référence THS '$data[26]' ".$repertoireImage.$data[23].".jpg, ".$repertoireImage.$data[26].".jpg et à la référence '$data[0]' ".$repertoireImage.$data[0].".jpg");
								}
							}
						}
						
						fclose($handle);
					}

					// Vérification que tous les articles ont une famille
					if (($handle = fopen($fichier, "r")) !== FALSE) {
						while (($data = fgetcsv($handle, 0, ";")) !== FALSE) {
							if ($data[27] == "") {
								throw new Exception("La référence THS '$data[26]' ne contient pas de famille");
							}

							if (!isset($listeFamilleArticle[strtolower($data[27])])) {
								throw new Exception("La référence THS '$data[26]' contient une famille qui n'existe pas");
							}
						}
						
						fclose($handle);
					}

					// Vérification des références
					$listeReference = array();
					$listeReferenceTHS = array();
					if (($handle = fopen($fichier, "r")) !== FALSE)
					{
						while (($data = fgetcsv($handle, 0, ";")) !== FALSE)
						{
							$listeReferenceTHS[] = $data[26];
							$listeReference[$data[26]] = $data[3];
						}
						fclose($handle);
					}

					try
					{
						$connection = Doctrine_Manager::connection();
						$connection->beginTransaction();

						if(sizeof($listeReferenceTHS) > 0)
						{
							// On récupère toutes les références existantes et on compare le couple reference_ths / reference_ths_sage
							// Si un couple ne correspond, on termine l'import
							$listeArticle = ArticleTable::recupererListeArticlePourImport($listeReferenceTHS);
							foreach($listeArticle as $article)
							{
								if($listeReference[$article->reference_ths] != $article->reference_ths_sage)
								{
									throw new Exception("La référence THS '$article->reference_ths' ne contient pas la même référence sage");
								}
							}
						}

						if (($handle = fopen($fichier, "r")) !== FALSE)
						{
							while (($data = fgetcsv($handle, 0, ";")) !== FALSE)
							{
								// On vérifie si le fournisseur existe
								$fournisseur = Doctrine_Core::getTable("Fournisseur")->findOneByCode($data[1]);
								
								if (!($fournisseur instanceof Fournisseur)) {
								    throw new Exception("Le code fournisseur '$data[1]' n'existent pas en base de données");
								}
								
								// Vérification champs obligatoire
								// A, B, C, D, E, F, G, K
								if ($data[0] == "" 
								    || $data[1] == ""
								    || $data[2] == ""
								    || $data[3] == ""
								    || $data[4] == ""
								    || $data[5] == ""
								    || $data[6] == ""
								    || $data[10] == ""
								) {
								    continue;
								}

								// On récupère le coefficient
								$prix = Utils::doubleFrToEn($data[11]);
								$coefficient = FournisseurCoefficientTable::findCoefficientByCodeFournisseurAndPrice($data[1], $prix);

								if (count($coefficient) > 1) {
								    throw new Exception("Il y a plusieurs coefficient pour le fournisseur ".$data[1]." et le montant ".$data[11]);
								} elseif (count($coefficient) == 1) {
								    $coeff = $coefficient[0]["coefficient"];
								} else {
								    $coeff = 1;
								}

								// Calcul du coeff de vente ths, marge ths en % et €
								/**
								 * H = F - ( F * G / 100 )
								 * 7 = 5 - ( 5 * 6 / 100 )
								 */
								$f = Utils::doubleFrToEn($data[5]);
								$g = Utils::doubleFrToEn($data[6]);
								$data[7] = Utils::doubleEnToFr(round($f-($f*$g/100), 2));
								$h = Utils::doubleFrToEn($data[7]);

								$data[8] = Utils::doubleEnToFr(round($prix/$h, 2));
								$data[12] = Utils::doubleEnToFr(round($prix-$h, 2));
								$data[9] = Utils::doubleEnToFr(round(Utils::doubleFrToEn($data[12])*100/$prix, 2));

								// Calcul des données avec le coefficient de vente THS
								$prix_web = $prix * $coeff;
								$data[14] = Utils::doubleEnToFr(round($prix_web, 2));
								$o = round($prix_web, 2);

								/**
								 * P = O / H
								 * 15 = 14 / 7;
								 */
								$data[15] = Utils::doubleEnToFr(round($o/$h, 2));

								/**
								 * R = O - H
								 * 17 = 14 - 7
								 */
								$data[17] = Utils::doubleEnToFr(round($o-$h, 2));
								$r = round($o-$h, 2);

								/**
								 * Q = R x 100 / O 
								 * 16 = 17 * 100 / 14
								 */
								$data[16] = Utils::doubleEnToFr(round($r*100/$o, 2));
				    
								$objet = Doctrine_Core::getTable("Article")->find($data[26]);
								
								if (!($objet instanceof Article)) {
									$objet = Doctrine_Core::getTable("Article")->findOneByReferenceThsSage($data[3]);
									
									if (!($objet instanceof Article)) {
										$objet = new Article();
										$objet->reference_ths = $data[26];
										$objet->reference_ths_sage = $data[3];
										$objet->reference_ths_affichage = $data[25];
										$objet->reference_ths_url = Utils::slugify($data[24]);
										$objet->id_famille = $listeFamilleArticle[strtolower($data[27])];
										$objet->designation_courte = $data[29];
										$objet->designation_detaillee = $data[28];
										$objet->specificite = $data[31];
										$objet->dimensions_a = $data[36];
										$objet->dimensions_b = $data[37];
										$objet->matiere = $data[39];
										$objet->pression_maximum = $data[40];
										$objet->temperature_maximum = $data[41];
										$objet->diametre_nominal = $data[38];
										$objet->reference_metrique_entree = $data[42];
										$objet->sexe_entree = $data[43];
										$objet->precision_supplementaire_entree = $data[44];
										$objet->reference_metrique_sortie = $data[45];
										$objet->sexe_sortie = $data[46];
										$objet->precision_supplementaire_sortie = $data[47];
										$objet->calibre_piece = $data[49];
										$objet->debit_max = $data[50];
										$objet->hors_gabarit = $data[51];
										$objet->piece_origine = $data[54];
										$objet->piece_visible = $data[55];
										$objet->poids = str_replace(",", ".", $data[35]);
										$objet->prix_vente_ht = str_replace(",", ".", $data[14]);
										$objet->taux_remise_fournisseur = str_replace(",", ".", $data[6]);
										$objet->prix_vente_ths = str_replace(",", ".", $data[14]);
										$objet->marge_pourcentage = str_replace(",", ".", $data[16]);
										$objet->marge_euro = str_replace(",", ".", $data[17]);
										$objet->prix_ttc = str_replace(",", ".", $data[14]) * 1.2;
										$objet->stock = $data[13];
										$objet->url_article = "http://www.haute-pression.com/".Utils::slugify($data[27])."/".Utils::slugify($data[24]).".html";
										$objet->url_image = $data[23];
										$objet->balise_title = $data[29];
										$objet->balise_meta = strtolower($data[27]).' - '.$data[28];
										$objet->minimum_commande = $data[52];
										$objet->save();
									} else {
									    if ($objet->reference_ths == $objet->reference_ths_sage) {
										$objet->reference_ths_affichage = $data[25];
										$objet->prix_vente_ht = str_replace(",", ".", $data[14]);
										$objet->taux_remise_fournisseur = str_replace(",", ".", $data[6]);
										$objet->prix_vente_ths = str_replace(",", ".", $data[14]);
										$objet->marge_pourcentage = str_replace(",", ".", $data[16]);
										$objet->marge_euro = str_replace(",", ".", $data[17]);
										$objet->prix_ttc = str_replace(",", ".", $data[14]) * 1.2;
										$objet->save();
									    }
									}
								} else {
								    if ($objet->reference_ths == $objet->reference_ths_sage) {
									$objet->reference_ths_affichage = $data[25];
									$objet->prix_vente_ht = str_replace(",", ".", $data[14]);
									$objet->taux_remise_fournisseur = str_replace(",", ".", $data[6]);
									$objet->prix_vente_ths = str_replace(",", ".", $data[14]);
									$objet->marge_pourcentage = str_replace(",", ".", $data[16]);
									$objet->marge_euro = str_replace(",", ".", $data[17]);
									$objet->prix_ttc = str_replace(",", ".", $data[14]) * 1.2;
									$objet->save();
								    }
								}
							}
							
							fclose($handle);
						}
						
						$connection->commit();
					}
					catch(Exception $e)
					{
						$connection->rollback();
						throw new Exception($e->getMessage().$e->getTraceAsString());
					}

					// Création des miniatures
					if (($handle = fopen($fichier, "r")) !== FALSE)
					{
						while (($data = fgetcsv($handle, 0, ";")) !== FALSE)
						{
						    if (file_exists($repertoireImage.$data[26].".jpg")) {
							// Création de la miniature
							$thumbnail = new sfThumbnail(100);
							$thumbnail->loadFile($repertoireImage.$data[26].".jpg");
							$thumbnail->save($repertoireArticleMiniature.$data[26]."_100.jpg");

							// on déplace les originaux
							copy($repertoireImage.$data[26].".jpg", $repertoireArticle.$data[26].".jpg");

							// unlink($repertoireImage.$data[26].".jpg");
						    }
						    
						    if (file_exists($repertoireImage.$data[0].".jpg")) {
							// Création de la miniature
							$thumbnail = new sfThumbnail(100);
							$thumbnail->loadFile($repertoireImage.$data[0].".jpg");
							$thumbnail->save($repertoireArticleMiniature.$data[0]."_100.jpg");

							// on déplace les originaux
							copy($repertoireImage.$data[0].".jpg", $repertoireArticle.$data[0].".jpg");

							// unlink($repertoireImage.$data[26].".jpg");
						    }
						}
						fclose($handle);
					}

					$this->getUser()->setFlash('notice', 'L\'importation s\'est déroulé avec succès');
				}
				catch(Exception $e)
				{
					unlink(sfConfig::get('sf_upload_dir').'/'.$filename.$extension);
					$this->getUser()->setFlash('error', $e->getMessage().$e->getTraceAsString());
					$this->redirect("@article");
				}
			}
			else
			{
				$this->getUser()->setFlash('error', 'Erreur lors de l\'importation');
			}
		}
		unlink(sfConfig::get('sf_upload_dir').'/'.$filename.$extension);
		$this->redirect("@article");
	}
	
	public function executeImport(sfWebRequest $request)
	{
		if ($request->isMethod("POST")) {
			$form = new ArticleImportForm();
			$form->bind($request->getParameter('article_import'), $request->getFiles('article_import'));
			if($form->isValid())
			{
				$file = $form->getValue('file');

				$filename = 'uploaded_'.sha1($file->getOriginalName());
				$extension = $file->getExtension($file->getOriginalExtension());
				$file->save(sfConfig::get('sf_upload_dir').'/'.$filename.$extension);

				// Récupère la liste des familles d'articles
				$listeFamilleArticle = FamilleTable::recupererListeArticleFamilleEnTableau();

				//Récupère les données
				$fichier = sfConfig::get('sf_upload_dir').'/'.$filename.$extension;

				try
				{
					// Vérification que les images existent bien
					$repertoireImage = sfConfig::get('import_dir').DIRECTORY_SEPARATOR;
					$repertoireArticle = sfConfig::get('article_image_dir').DIRECTORY_SEPARATOR;
					$repertoireArticleMiniature = sfConfig::get('article_miniature_dir').DIRECTORY_SEPARATOR;

					if (($handle = fopen($fichier, "r")) !== FALSE) {
						while (($data = fgetcsv($handle, 0, ";")) !== FALSE) {
							if (!file_exists($repertoireImage.$data[26].".jpg")) {
								throw new Exception("Aucune image n'est associé à la référence THS '$data[26]' ".$repertoireImage.$data[26].".jpg");
							}
						}
						
						fclose($handle);
					}

					// Vérification que tous les articles ont une famille
					if (($handle = fopen($fichier, "r")) !== FALSE) {
						while (($data = fgetcsv($handle, 0, ";")) !== FALSE) {
							if ($data[27] == "") {
								throw new Exception("La référence THS '$data[26]' ne contient pas de famille");
							}

							if (!isset($listeFamilleArticle[strtolower($data[27])])) {
								throw new Exception("La référence THS '$data[26]' contient une famille qui n'existe pas");
							}
						}
						
						fclose($handle);
					}

					// Vérification des références
					$listeReference = array();
					$listeReferenceTHS = array();
					if (($handle = fopen($fichier, "r")) !== FALSE)
					{
						while (($data = fgetcsv($handle, 0, ";")) !== FALSE)
						{
							$listeReferenceTHS[] = $data[26];
							$listeReference[$data[26]] = $data[3];
						}
						fclose($handle);
					}

					try
					{
						$connection = Doctrine_Manager::connection();
						$connection->beginTransaction();

						if(sizeof($listeReferenceTHS) > 0)
						{
							// On récupère toutes les références existantes et on compare le couple reference_ths / reference_ths_sage
							// Si un couple ne correspond, on termine l'import
							$listeArticle = ArticleTable::recupererListeArticlePourImport($listeReferenceTHS);
							foreach($listeArticle as $article)
							{
								if($listeReference[$article->reference_ths] != $article->reference_ths_sage)
								{
									throw new Exception("La référence THS '$article->reference_ths' ne contient pas la même référence sage");
								}
							}
						}

						if (($handle = fopen($fichier, "r")) !== FALSE)
						{
							while (($data = fgetcsv($handle, 0, ";")) !== FALSE)
							{
								$objet = Doctrine_Core::getTable("Article")->find($data[26]);
								
								if(!($objet instanceof Article))
								{
									$objet = new Article();
								}
								
								$objet->reference_ths = $data[26];
								$objet->reference_ths_sage = $data[3];
								$objet->id_famille = $listeFamilleArticle[strtolower($data[27])];
								$objet->designation_courte = $data[29];
								$objet->designation_detaillee = $data[28];
								$objet->specificite = $data[31];
								$objet->dimensions_a = $data[36];
								$objet->dimensions_b = $data[37];
								$objet->matiere = $data[39];
								$objet->pression_maximum = $data[40];
								$objet->temperature_maximum = $data[41];
								$objet->diametre_nominal = $data[38];
								$objet->reference_metrique_entree = $data[42];
								$objet->sexe_entree = $data[43];
								$objet->precision_supplementaire_entree = $data[44];
								$objet->reference_metrique_sortie = $data[45];
								$objet->sexe_sortie = $data[46];
								$objet->precision_supplementaire_sortie = $data[47];
								$objet->calibre_piece = $data[49];
								$objet->debit_max = $data[50];
								$objet->hors_gabarit = $data[51];
								$objet->piece_origine = $data[54];
								$objet->piece_visible = $data[55];
								$objet->poids = str_replace(",", ".", $data[35]);
								$objet->prix_vente_ht = str_replace(",", ".", $data[11]);
								$objet->taux_remise_fournisseur = str_replace(",", ".", $data[6]);
								$objet->prix_vente_ths = str_replace(",", ".", $data[11]);
								$objet->marge_pourcentage = str_replace(",", ".", $data[9]);
								$objet->marge_euro = str_replace(",", ".", $data[12]);
								$objet->prix_ttc = str_replace(",", ".", $data[11]) * 1.2;
								$objet->stock = $data[13];
								$objet->url_article = "http://www.haute-pression.com/".Utils::slugify($data[27])."/".Utils::slugify($data[26]).".html";
								$objet->balise_title = $data[29];
								$objet->balise_meta = strtolower($data[27]).' - '.$data[28];
								$objet->minimum_commande = $data[52];
								$objet->save();
							}
							
							fclose($handle);
						}
						
						$connection->commit();
					}
					catch(Exception $e)
					{
						$connection->rollback();
						throw new Exception($e->getMessage());
					}

					// Création des miniatures
					if (($handle = fopen($fichier, "r")) !== FALSE)
					{
						while (($data = fgetcsv($handle, 0, ";")) !== FALSE)
						{
							// Création de la miniature
							$thumbnail = new sfThumbnail(100);
							$thumbnail->loadFile($repertoireImage.$data[26].".jpg");
							$thumbnail->save($repertoireArticleMiniature.$data[26]."_100.jpg");

							// on déplace les originaux
							copy($repertoireImage.$data[26].".jpg", $repertoireArticle.$data[26].".jpg");

							// unlink($repertoireImage.$data[26].".jpg");
						}
						fclose($handle);
					}

					$this->getUser()->setFlash('notice', 'L\'importation s\'est déroulé avec succès');
				}
				catch(Exception $e)
				{
					unlink(sfConfig::get('sf_upload_dir').'/'.$filename.$extension);
					$this->getUser()->setFlash('error', $e->getMessage());
					$this->redirect("@article");
				}
			}
			else
			{
				$this->getUser()->setFlash('error', 'Erreur lors de l\'importation');
			}
		}
		unlink(sfConfig::get('sf_upload_dir').'/'.$filename.$extension);
		$this->redirect("@article");
	}

	public function executeBatch(sfWebRequest $request)
	{
		$request->checkCSRFProtection();

		if($request->hasParameter("max_per_page_article") && !$action = $request->getParameter('batch_action'))
		{
			$this->getUser()->setAttribute('max_per_page_article', $request->getParameter("max_per_page_article"));
			$this->redirect('@article');
		}

		if (!$ids = $request->getParameter('ids'))
		{
			$this->getUser()->setFlash('error', 'You must at least select one item.');

			$this->redirect('@article');
		}

		if (!$action = $request->getParameter('batch_action'))
		{
			$this->getUser()->setFlash('error', 'You must select an action to execute on the selected items.');

			$this->redirect('@article');
		}

		if (!method_exists($this, $method = 'execute'.ucfirst($action)))
		{
			throw new InvalidArgumentException(sprintf('You must create a "%s" method for action "%s"', $method, $action));
		}

		if (!$this->getUser()->hasCredential($this->configuration->getCredentials($action)))
		{
			$this->forward(sfConfig::get('sf_secure_module'), sfConfig::get('sf_secure_action'));
		}

		$validator = new sfValidatorDoctrineChoice(array('multiple' => true, 'model' => 'Article'));
		try
		{
			// validate ids
			$ids = $validator->clean($ids);

			// execute batch
			$this->$method($request);
		}
		catch (sfValidatorError $e)
		{
			$this->getUser()->setFlash('error', 'A problem occurs when deleting the selected items as some items do not exist anymore.');
		}

		$this->redirect('@article');
	}
	
	public function executeRecupererArticlePourAssociation(sfWebRequest $request)
	{
		$reference_ths_sage = $request->getPostParameter('reference_ths_sage');
		if ($request->isXmlHttpRequest())
		{
			$donnees = array();
			$donnees['reference_ths_sage'] = "";
			$donnees['reference_ths'] = "";
			$donnees['designation'] = "";
			$donnees['prix'] = "";
			$donnees['poids'] = "";
			
			$article = Doctrine::getTable("Article")->findOneByReferenceThsSage($reference_ths_sage);
			
			if($article instanceof Article)
			{
				$donnees = array();
				$donnees['reference_ths_sage'] = $article->getReferenceThsSage();
				$donnees['reference_ths'] = $article->getReferenceThs();
				$donnees['designation'] = $article->getDesignationCourte();
				$donnees['prix'] = $article->getPrixVenteThs();
				$donnees['poids'] = $article->getPoids();
			}
			else
			{
				$article = Doctrine::getTable("Article")->findOneByReferenceThs($reference_ths_sage);
				
				if($article instanceof Article)
				{
					$donnees = array();
					$donnees['reference_ths_sage'] = $article->getReferenceThsSage();
					$donnees['reference_ths'] = $article->getReferenceThs();
					$donnees['designation'] = $article->getDesignationCourte();
					$donnees['prix'] = $article->getPrixVenteThs();
					$donnees['poids'] = $article->getPoids();
				}
			}
			
			if(!($article instanceof Article))
			{
				$nettoyeur = Doctrine::getTable("NettoyeurHautePression")->findOneByReferenceThs($reference_ths_sage);
				
				if($nettoyeur instanceof NettoyeurHautePression)
				{
					$donnees = array();
					$donnees['reference_ths_sage'] = $nettoyeur->getReferenceConstructeur();
					$donnees['reference_ths'] = $nettoyeur->getReferenceThs();
					$donnees['designation'] = $nettoyeur->getMarque()->getLibelle()." ".$nettoyeur->getModele();
					$donnees['prix'] = $nettoyeur->getPrixVenteThs();
					$donnees['poids'] = $nettoyeur->getPoids();
				}
				else
				{
					$nettoyeur = Doctrine::getTable("NettoyeurHautePression")->findOneByReferenceConstructeur($reference_ths_sage);
				
					if($nettoyeur instanceof NettoyeurHautePression)
					{
						$donnees = array();
						$donnees['reference_ths_sage'] = $nettoyeur->getReferenceConstructeur();
						$donnees['reference_ths'] = $nettoyeur->getReferenceThs();
						$donnees['designation'] = $nettoyeur->getMarque()->getLibelle()." ".$nettoyeur->getModele();
						$donnees['prix'] = $nettoyeur->getPrixVenteThs();
						$donnees['poids'] = $nettoyeur->getPoids();
					}
				}
			}
			
			return $this->renderText(json_encode($donnees));
		}
		return sfView::NONE;
	}
}
